"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import feast.core.DataSource_pb2
import feast.core.Entity_pb2
import feast.core.FeatureService_pb2
import feast.core.FeatureView_pb2
import feast.core.InfraObject_pb2
import feast.core.OnDemandFeatureView_pb2
import feast.core.Permission_pb2
import feast.core.Project_pb2
import feast.core.Registry_pb2
import feast.core.SavedDataset_pb2
import feast.core.StreamFeatureView_pb2
import feast.core.ValidationProfile_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class PaginationParams(google.protobuf.message.Message):
    """Common pagination and sorting messages"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAGE_FIELD_NUMBER: builtins.int
    LIMIT_FIELD_NUMBER: builtins.int
    page: builtins.int
    """1-based page number"""
    limit: builtins.int
    """Number of items per page"""
    def __init__(
        self,
        *,
        page: builtins.int = ...,
        limit: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["limit", b"limit", "page", b"page"]) -> None: ...

global___PaginationParams = PaginationParams

class SortingParams(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SORT_BY_FIELD_NUMBER: builtins.int
    SORT_ORDER_FIELD_NUMBER: builtins.int
    sort_by: builtins.str
    """Field to sort by (supports dot notation)"""
    sort_order: builtins.str
    """"asc" or "desc" """
    def __init__(
        self,
        *,
        sort_by: builtins.str = ...,
        sort_order: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["sort_by", b"sort_by", "sort_order", b"sort_order"]) -> None: ...

global___SortingParams = SortingParams

class PaginationMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAGE_FIELD_NUMBER: builtins.int
    LIMIT_FIELD_NUMBER: builtins.int
    TOTAL_COUNT_FIELD_NUMBER: builtins.int
    TOTAL_PAGES_FIELD_NUMBER: builtins.int
    HAS_NEXT_FIELD_NUMBER: builtins.int
    HAS_PREVIOUS_FIELD_NUMBER: builtins.int
    page: builtins.int
    limit: builtins.int
    total_count: builtins.int
    total_pages: builtins.int
    has_next: builtins.bool
    has_previous: builtins.bool
    def __init__(
        self,
        *,
        page: builtins.int = ...,
        limit: builtins.int = ...,
        total_count: builtins.int = ...,
        total_pages: builtins.int = ...,
        has_next: builtins.bool = ...,
        has_previous: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["has_next", b"has_next", "has_previous", b"has_previous", "limit", b"limit", "page", b"page", "total_count", b"total_count", "total_pages", b"total_pages"]) -> None: ...

global___PaginationMetadata = PaginationMetadata

class RefreshRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    project: builtins.str
    def __init__(
        self,
        *,
        project: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project", b"project"]) -> None: ...

global___RefreshRequest = RefreshRequest

class UpdateInfraRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INFRA_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def infra(self) -> feast.core.InfraObject_pb2.Infra: ...
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        infra: feast.core.InfraObject_pb2.Infra | None = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["infra", b"infra"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "infra", b"infra", "project", b"project"]) -> None: ...

global___UpdateInfraRequest = UpdateInfraRequest

class GetInfraRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "project", b"project"]) -> None: ...

global___GetInfraRequest = GetInfraRequest

class ListProjectMetadataRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "project", b"project"]) -> None: ...

global___ListProjectMetadataRequest = ListProjectMetadataRequest

class ListProjectMetadataResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_METADATA_FIELD_NUMBER: builtins.int
    @property
    def project_metadata(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Registry_pb2.ProjectMetadata]: ...
    def __init__(
        self,
        *,
        project_metadata: collections.abc.Iterable[feast.core.Registry_pb2.ProjectMetadata] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project_metadata", b"project_metadata"]) -> None: ...

global___ListProjectMetadataResponse = ListProjectMetadataResponse

class ApplyMaterializationRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_VIEW_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    START_DATE_FIELD_NUMBER: builtins.int
    END_DATE_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def feature_view(self) -> feast.core.FeatureView_pb2.FeatureView: ...
    project: builtins.str
    @property
    def start_date(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def end_date(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    commit: builtins.bool
    def __init__(
        self,
        *,
        feature_view: feast.core.FeatureView_pb2.FeatureView | None = ...,
        project: builtins.str = ...,
        start_date: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        end_date: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["end_date", b"end_date", "feature_view", b"feature_view", "start_date", b"start_date"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "end_date", b"end_date", "feature_view", b"feature_view", "project", b"project", "start_date", b"start_date"]) -> None: ...

global___ApplyMaterializationRequest = ApplyMaterializationRequest

class ApplyEntityRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENTITY_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def entity(self) -> feast.core.Entity_pb2.Entity: ...
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        entity: feast.core.Entity_pb2.Entity | None = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["entity", b"entity"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "entity", b"entity", "project", b"project"]) -> None: ...

global___ApplyEntityRequest = ApplyEntityRequest

class GetEntityRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetEntityRequest = GetEntityRequest

class ListEntitiesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListEntitiesRequest = ListEntitiesRequest

class ListEntitiesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENTITIES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def entities(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Entity_pb2.Entity]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        entities: collections.abc.Iterable[feast.core.Entity_pb2.Entity] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["entities", b"entities", "pagination", b"pagination"]) -> None: ...

global___ListEntitiesResponse = ListEntitiesResponse

class DeleteEntityRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "name", b"name", "project", b"project"]) -> None: ...

global___DeleteEntityRequest = DeleteEntityRequest

class ApplyDataSourceRequest(google.protobuf.message.Message):
    """DataSources"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCE_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def data_source(self) -> feast.core.DataSource_pb2.DataSource: ...
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        data_source: feast.core.DataSource_pb2.DataSource | None = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["data_source", b"data_source"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "data_source", b"data_source", "project", b"project"]) -> None: ...

global___ApplyDataSourceRequest = ApplyDataSourceRequest

class GetDataSourceRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetDataSourceRequest = GetDataSourceRequest

class ListDataSourcesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListDataSourcesRequest = ListDataSourcesRequest

class ListDataSourcesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def data_sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.DataSource_pb2.DataSource]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        data_sources: collections.abc.Iterable[feast.core.DataSource_pb2.DataSource] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_sources", b"data_sources", "pagination", b"pagination"]) -> None: ...

global___ListDataSourcesResponse = ListDataSourcesResponse

class DeleteDataSourceRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "name", b"name", "project", b"project"]) -> None: ...

global___DeleteDataSourceRequest = DeleteDataSourceRequest

class ApplyFeatureViewRequest(google.protobuf.message.Message):
    """FeatureViews"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_VIEW_FIELD_NUMBER: builtins.int
    ON_DEMAND_FEATURE_VIEW_FIELD_NUMBER: builtins.int
    STREAM_FEATURE_VIEW_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def feature_view(self) -> feast.core.FeatureView_pb2.FeatureView: ...
    @property
    def on_demand_feature_view(self) -> feast.core.OnDemandFeatureView_pb2.OnDemandFeatureView: ...
    @property
    def stream_feature_view(self) -> feast.core.StreamFeatureView_pb2.StreamFeatureView: ...
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        feature_view: feast.core.FeatureView_pb2.FeatureView | None = ...,
        on_demand_feature_view: feast.core.OnDemandFeatureView_pb2.OnDemandFeatureView | None = ...,
        stream_feature_view: feast.core.StreamFeatureView_pb2.StreamFeatureView | None = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["base_feature_view", b"base_feature_view", "feature_view", b"feature_view", "on_demand_feature_view", b"on_demand_feature_view", "stream_feature_view", b"stream_feature_view"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_feature_view", b"base_feature_view", "commit", b"commit", "feature_view", b"feature_view", "on_demand_feature_view", b"on_demand_feature_view", "project", b"project", "stream_feature_view", b"stream_feature_view"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["base_feature_view", b"base_feature_view"]) -> typing_extensions.Literal["feature_view", "on_demand_feature_view", "stream_feature_view"] | None: ...

global___ApplyFeatureViewRequest = ApplyFeatureViewRequest

class GetFeatureViewRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetFeatureViewRequest = GetFeatureViewRequest

class ListFeatureViewsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListFeatureViewsRequest = ListFeatureViewsRequest

class ListFeatureViewsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_VIEWS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def feature_views(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.FeatureView_pb2.FeatureView]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        feature_views: collections.abc.Iterable[feast.core.FeatureView_pb2.FeatureView] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_views", b"feature_views", "pagination", b"pagination"]) -> None: ...

global___ListFeatureViewsResponse = ListFeatureViewsResponse

class DeleteFeatureViewRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "name", b"name", "project", b"project"]) -> None: ...

global___DeleteFeatureViewRequest = DeleteFeatureViewRequest

class AnyFeatureView(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_VIEW_FIELD_NUMBER: builtins.int
    ON_DEMAND_FEATURE_VIEW_FIELD_NUMBER: builtins.int
    STREAM_FEATURE_VIEW_FIELD_NUMBER: builtins.int
    @property
    def feature_view(self) -> feast.core.FeatureView_pb2.FeatureView: ...
    @property
    def on_demand_feature_view(self) -> feast.core.OnDemandFeatureView_pb2.OnDemandFeatureView: ...
    @property
    def stream_feature_view(self) -> feast.core.StreamFeatureView_pb2.StreamFeatureView: ...
    def __init__(
        self,
        *,
        feature_view: feast.core.FeatureView_pb2.FeatureView | None = ...,
        on_demand_feature_view: feast.core.OnDemandFeatureView_pb2.OnDemandFeatureView | None = ...,
        stream_feature_view: feast.core.StreamFeatureView_pb2.StreamFeatureView | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["any_feature_view", b"any_feature_view", "feature_view", b"feature_view", "on_demand_feature_view", b"on_demand_feature_view", "stream_feature_view", b"stream_feature_view"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["any_feature_view", b"any_feature_view", "feature_view", b"feature_view", "on_demand_feature_view", b"on_demand_feature_view", "stream_feature_view", b"stream_feature_view"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["any_feature_view", b"any_feature_view"]) -> typing_extensions.Literal["feature_view", "on_demand_feature_view", "stream_feature_view"] | None: ...

global___AnyFeatureView = AnyFeatureView

class GetAnyFeatureViewRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetAnyFeatureViewRequest = GetAnyFeatureViewRequest

class GetAnyFeatureViewResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ANY_FEATURE_VIEW_FIELD_NUMBER: builtins.int
    @property
    def any_feature_view(self) -> global___AnyFeatureView: ...
    def __init__(
        self,
        *,
        any_feature_view: global___AnyFeatureView | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["any_feature_view", b"any_feature_view"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["any_feature_view", b"any_feature_view"]) -> None: ...

global___GetAnyFeatureViewResponse = GetAnyFeatureViewResponse

class ListAllFeatureViewsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    ENTITY_FIELD_NUMBER: builtins.int
    FEATURE_FIELD_NUMBER: builtins.int
    FEATURE_SERVICE_FIELD_NUMBER: builtins.int
    DATA_SOURCE_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    entity: builtins.str
    feature: builtins.str
    feature_service: builtins.str
    data_source: builtins.str
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        entity: builtins.str = ...,
        feature: builtins.str = ...,
        feature_service: builtins.str = ...,
        data_source: builtins.str = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "data_source", b"data_source", "entity", b"entity", "feature", b"feature", "feature_service", b"feature_service", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListAllFeatureViewsRequest = ListAllFeatureViewsRequest

class ListAllFeatureViewsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_VIEWS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def feature_views(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AnyFeatureView]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        feature_views: collections.abc.Iterable[global___AnyFeatureView] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_views", b"feature_views", "pagination", b"pagination"]) -> None: ...

global___ListAllFeatureViewsResponse = ListAllFeatureViewsResponse

class GetStreamFeatureViewRequest(google.protobuf.message.Message):
    """StreamFeatureView"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetStreamFeatureViewRequest = GetStreamFeatureViewRequest

class ListStreamFeatureViewsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListStreamFeatureViewsRequest = ListStreamFeatureViewsRequest

class ListStreamFeatureViewsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAM_FEATURE_VIEWS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def stream_feature_views(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.StreamFeatureView_pb2.StreamFeatureView]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        stream_feature_views: collections.abc.Iterable[feast.core.StreamFeatureView_pb2.StreamFeatureView] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "stream_feature_views", b"stream_feature_views"]) -> None: ...

global___ListStreamFeatureViewsResponse = ListStreamFeatureViewsResponse

class GetOnDemandFeatureViewRequest(google.protobuf.message.Message):
    """OnDemandFeatureView"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetOnDemandFeatureViewRequest = GetOnDemandFeatureViewRequest

class ListOnDemandFeatureViewsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListOnDemandFeatureViewsRequest = ListOnDemandFeatureViewsRequest

class ListOnDemandFeatureViewsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ON_DEMAND_FEATURE_VIEWS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def on_demand_feature_views(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.OnDemandFeatureView_pb2.OnDemandFeatureView]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        on_demand_feature_views: collections.abc.Iterable[feast.core.OnDemandFeatureView_pb2.OnDemandFeatureView] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["on_demand_feature_views", b"on_demand_feature_views", "pagination", b"pagination"]) -> None: ...

global___ListOnDemandFeatureViewsResponse = ListOnDemandFeatureViewsResponse

class ApplyFeatureServiceRequest(google.protobuf.message.Message):
    """FeatureServices"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SERVICE_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def feature_service(self) -> feast.core.FeatureService_pb2.FeatureService: ...
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        feature_service: feast.core.FeatureService_pb2.FeatureService | None = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["feature_service", b"feature_service"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "feature_service", b"feature_service", "project", b"project"]) -> None: ...

global___ApplyFeatureServiceRequest = ApplyFeatureServiceRequest

class GetFeatureServiceRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetFeatureServiceRequest = GetFeatureServiceRequest

class ListFeatureServicesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    FEATURE_VIEW_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    feature_view: builtins.str
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        feature_view: builtins.str = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "feature_view", b"feature_view", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListFeatureServicesRequest = ListFeatureServicesRequest

class ListFeatureServicesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SERVICES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def feature_services(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.FeatureService_pb2.FeatureService]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        feature_services: collections.abc.Iterable[feast.core.FeatureService_pb2.FeatureService] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_services", b"feature_services", "pagination", b"pagination"]) -> None: ...

global___ListFeatureServicesResponse = ListFeatureServicesResponse

class DeleteFeatureServiceRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "name", b"name", "project", b"project"]) -> None: ...

global___DeleteFeatureServiceRequest = DeleteFeatureServiceRequest

class ApplySavedDatasetRequest(google.protobuf.message.Message):
    """SavedDataset"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SAVED_DATASET_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def saved_dataset(self) -> feast.core.SavedDataset_pb2.SavedDataset: ...
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        saved_dataset: feast.core.SavedDataset_pb2.SavedDataset | None = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["saved_dataset", b"saved_dataset"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "project", b"project", "saved_dataset", b"saved_dataset"]) -> None: ...

global___ApplySavedDatasetRequest = ApplySavedDatasetRequest

class GetSavedDatasetRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetSavedDatasetRequest = GetSavedDatasetRequest

class ListSavedDatasetsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListSavedDatasetsRequest = ListSavedDatasetsRequest

class ListSavedDatasetsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SAVED_DATASETS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def saved_datasets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.SavedDataset_pb2.SavedDataset]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        saved_datasets: collections.abc.Iterable[feast.core.SavedDataset_pb2.SavedDataset] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "saved_datasets", b"saved_datasets"]) -> None: ...

global___ListSavedDatasetsResponse = ListSavedDatasetsResponse

class DeleteSavedDatasetRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "name", b"name", "project", b"project"]) -> None: ...

global___DeleteSavedDatasetRequest = DeleteSavedDatasetRequest

class ApplyValidationReferenceRequest(google.protobuf.message.Message):
    """ValidationReference"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATION_REFERENCE_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def validation_reference(self) -> feast.core.ValidationProfile_pb2.ValidationReference: ...
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        validation_reference: feast.core.ValidationProfile_pb2.ValidationReference | None = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["validation_reference", b"validation_reference"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "project", b"project", "validation_reference", b"validation_reference"]) -> None: ...

global___ApplyValidationReferenceRequest = ApplyValidationReferenceRequest

class GetValidationReferenceRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetValidationReferenceRequest = GetValidationReferenceRequest

class ListValidationReferencesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListValidationReferencesRequest = ListValidationReferencesRequest

class ListValidationReferencesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALIDATION_REFERENCES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def validation_references(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.ValidationProfile_pb2.ValidationReference]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        validation_references: collections.abc.Iterable[feast.core.ValidationProfile_pb2.ValidationReference] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "validation_references", b"validation_references"]) -> None: ...

global___ListValidationReferencesResponse = ListValidationReferencesResponse

class DeleteValidationReferenceRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "name", b"name", "project", b"project"]) -> None: ...

global___DeleteValidationReferenceRequest = DeleteValidationReferenceRequest

class ApplyPermissionRequest(google.protobuf.message.Message):
    """Permissions"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PERMISSION_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def permission(self) -> feast.core.Permission_pb2.Permission: ...
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        permission: feast.core.Permission_pb2.Permission | None = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["permission", b"permission"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "permission", b"permission", "project", b"project"]) -> None: ...

global___ApplyPermissionRequest = ApplyPermissionRequest

class GetPermissionRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name", "project", b"project"]) -> None: ...

global___GetPermissionRequest = GetPermissionRequest

class ListPermissionsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "pagination", b"pagination", "project", b"project", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListPermissionsRequest = ListPermissionsRequest

class ListPermissionsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PERMISSIONS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def permissions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Permission_pb2.Permission]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        permissions: collections.abc.Iterable[feast.core.Permission_pb2.Permission] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "permissions", b"permissions"]) -> None: ...

global___ListPermissionsResponse = ListPermissionsResponse

class DeletePermissionRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    name: builtins.str
    project: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "name", b"name", "project", b"project"]) -> None: ...

global___DeletePermissionRequest = DeletePermissionRequest

class ApplyProjectRequest(google.protobuf.message.Message):
    """Projects"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    @property
    def project(self) -> feast.core.Project_pb2.Project: ...
    commit: builtins.bool
    def __init__(
        self,
        *,
        project: feast.core.Project_pb2.Project | None = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["project", b"project"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "project", b"project"]) -> None: ...

global___ApplyProjectRequest = ApplyProjectRequest

class GetProjectRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    name: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "name", b"name"]) -> None: ...

global___GetProjectRequest = GetProjectRequest

class ListProjectsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    allow_cache: builtins.bool
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        allow_cache: builtins.bool = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "pagination", b"pagination", "sorting", b"sorting", "tags", b"tags"]) -> None: ...

global___ListProjectsRequest = ListProjectsRequest

class ListProjectsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECTS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def projects(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Project_pb2.Project]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        projects: collections.abc.Iterable[feast.core.Project_pb2.Project] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "projects", b"projects"]) -> None: ...

global___ListProjectsResponse = ListProjectsResponse

class DeleteProjectRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    name: builtins.str
    commit: builtins.bool
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        commit: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commit", b"commit", "name", b"name"]) -> None: ...

global___DeleteProjectRequest = DeleteProjectRequest

class EntityReference(google.protobuf.message.Message):
    """Lineage"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    type: builtins.str
    """"dataSource", "entity", "featureView", "featureService" """
    name: builtins.str
    def __init__(
        self,
        *,
        type: builtins.str = ...,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "type", b"type"]) -> None: ...

global___EntityReference = EntityReference

class EntityRelation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_FIELD_NUMBER: builtins.int
    TARGET_FIELD_NUMBER: builtins.int
    @property
    def source(self) -> global___EntityReference: ...
    @property
    def target(self) -> global___EntityReference: ...
    def __init__(
        self,
        *,
        source: global___EntityReference | None = ...,
        target: global___EntityReference | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["source", b"source", "target", b"target"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["source", b"source", "target", b"target"]) -> None: ...

global___EntityRelation = EntityRelation

class GetRegistryLineageRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    FILTER_OBJECT_TYPE_FIELD_NUMBER: builtins.int
    FILTER_OBJECT_NAME_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    allow_cache: builtins.bool
    filter_object_type: builtins.str
    filter_object_name: builtins.str
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        filter_object_type: builtins.str = ...,
        filter_object_name: builtins.str = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "filter_object_name", b"filter_object_name", "filter_object_type", b"filter_object_type", "pagination", b"pagination", "project", b"project", "sorting", b"sorting"]) -> None: ...

global___GetRegistryLineageRequest = GetRegistryLineageRequest

class GetRegistryLineageResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RELATIONSHIPS_FIELD_NUMBER: builtins.int
    INDIRECT_RELATIONSHIPS_FIELD_NUMBER: builtins.int
    RELATIONSHIPS_PAGINATION_FIELD_NUMBER: builtins.int
    INDIRECT_RELATIONSHIPS_PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def relationships(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___EntityRelation]: ...
    @property
    def indirect_relationships(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___EntityRelation]: ...
    @property
    def relationships_pagination(self) -> global___PaginationMetadata: ...
    @property
    def indirect_relationships_pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        relationships: collections.abc.Iterable[global___EntityRelation] | None = ...,
        indirect_relationships: collections.abc.Iterable[global___EntityRelation] | None = ...,
        relationships_pagination: global___PaginationMetadata | None = ...,
        indirect_relationships_pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["indirect_relationships_pagination", b"indirect_relationships_pagination", "relationships_pagination", b"relationships_pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["indirect_relationships", b"indirect_relationships", "indirect_relationships_pagination", b"indirect_relationships_pagination", "relationships", b"relationships", "relationships_pagination", b"relationships_pagination"]) -> None: ...

global___GetRegistryLineageResponse = GetRegistryLineageResponse

class GetObjectRelationshipsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    OBJECT_TYPE_FIELD_NUMBER: builtins.int
    OBJECT_NAME_FIELD_NUMBER: builtins.int
    INCLUDE_INDIRECT_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    object_type: builtins.str
    object_name: builtins.str
    include_indirect: builtins.bool
    allow_cache: builtins.bool
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        object_type: builtins.str = ...,
        object_name: builtins.str = ...,
        include_indirect: builtins.bool = ...,
        allow_cache: builtins.bool = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "include_indirect", b"include_indirect", "object_name", b"object_name", "object_type", b"object_type", "pagination", b"pagination", "project", b"project", "sorting", b"sorting"]) -> None: ...

global___GetObjectRelationshipsRequest = GetObjectRelationshipsRequest

class GetObjectRelationshipsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RELATIONSHIPS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def relationships(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___EntityRelation]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        relationships: collections.abc.Iterable[global___EntityRelation] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "relationships", b"relationships"]) -> None: ...

global___GetObjectRelationshipsResponse = GetObjectRelationshipsResponse

class Feature(google.protobuf.message.Message):
    """Feature messages"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    FEATURE_VIEW_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    CREATED_TIMESTAMP_FIELD_NUMBER: builtins.int
    LAST_UPDATED_TIMESTAMP_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    name: builtins.str
    feature_view: builtins.str
    type: builtins.str
    description: builtins.str
    owner: builtins.str
    @property
    def created_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def last_updated_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        feature_view: builtins.str = ...,
        type: builtins.str = ...,
        description: builtins.str = ...,
        owner: builtins.str = ...,
        created_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        last_updated_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["created_timestamp", b"created_timestamp", "last_updated_timestamp", b"last_updated_timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["created_timestamp", b"created_timestamp", "description", b"description", "feature_view", b"feature_view", "last_updated_timestamp", b"last_updated_timestamp", "name", b"name", "owner", b"owner", "tags", b"tags", "type", b"type"]) -> None: ...

global___Feature = Feature

class ListFeaturesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    FEATURE_VIEW_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    SORTING_FIELD_NUMBER: builtins.int
    project: builtins.str
    feature_view: builtins.str
    name: builtins.str
    allow_cache: builtins.bool
    @property
    def pagination(self) -> global___PaginationParams: ...
    @property
    def sorting(self) -> global___SortingParams: ...
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        feature_view: builtins.str = ...,
        name: builtins.str = ...,
        allow_cache: builtins.bool = ...,
        pagination: global___PaginationParams | None = ...,
        sorting: global___SortingParams | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination", "sorting", b"sorting"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "feature_view", b"feature_view", "name", b"name", "pagination", b"pagination", "project", b"project", "sorting", b"sorting"]) -> None: ...

global___ListFeaturesRequest = ListFeaturesRequest

class ListFeaturesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Feature]: ...
    @property
    def pagination(self) -> global___PaginationMetadata: ...
    def __init__(
        self,
        *,
        features: collections.abc.Iterable[global___Feature] | None = ...,
        pagination: global___PaginationMetadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination", b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["features", b"features", "pagination", b"pagination"]) -> None: ...

global___ListFeaturesResponse = ListFeaturesResponse

class GetFeatureRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    FEATURE_VIEW_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    ALLOW_CACHE_FIELD_NUMBER: builtins.int
    project: builtins.str
    feature_view: builtins.str
    name: builtins.str
    allow_cache: builtins.bool
    def __init__(
        self,
        *,
        project: builtins.str = ...,
        feature_view: builtins.str = ...,
        name: builtins.str = ...,
        allow_cache: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_cache", b"allow_cache", "feature_view", b"feature_view", "name", b"name", "project", b"project"]) -> None: ...

global___GetFeatureRequest = GetFeatureRequest



import logging
from typing import TYPE_CHECKING, Dict, List, Optional, Union, cast

from feast import FeatureView
from feast.infra.common.materialization_job import MaterializationTask
from feast.infra.common.retrieval_task import HistoricalRetrievalTask
from feast.infra.compute_engines.algorithms.topo import topological_sort
from feast.infra.compute_engines.dag.node import DAGNode
from feast.infra.compute_engines.dag.plan import ExecutionPlan
from feast.infra.compute_engines.feature_builder import FeatureBuilder
from feast.infra.compute_engines.ray.config import RayComputeEngineConfig
from feast.infra.compute_engines.ray.nodes import (
    RayAggregationNode,
    RayDedupNode,
    RayDerivedReadNode,
    RayFilterNode,
    RayJoinNode,
    RayReadNode,
    RayTransformationNode,
    RayWriteNode,
)

if TYPE_CHECKING:
    from feast.infra.compute_engines.ray.config import RayComputeEngineConfig

logger = logging.getLogger(__name__)


class RayFeatureBuilder(FeatureBuilder):
    """
    Ray-specific feature builder that constructs execution plans using Ray DAG nodes.
    This builder translates FeatureView definitions into Ray-optimized execution DAGs
    that can leverage distributed computing for large-scale feature processing.
    """

    def __init__(
        self,
        registry,
        feature_view,
        task: Union[MaterializationTask, HistoricalRetrievalTask],
        config: "RayComputeEngineConfig",
    ):
        super().__init__(registry, feature_view, task)
        self.config = config
        self.is_historical_retrieval = isinstance(task, HistoricalRetrievalTask)
        self.is_materialization = isinstance(task, MaterializationTask)

    def build_source_node(self, view):
        """Build the source node for reading feature data."""
        data_source = getattr(view, "batch_source", None) or getattr(
            view, "source", None
        )
        start_time = self.task.start_time
        end_time = self.task.end_time
        column_info = self.get_column_info(view)

        node = RayReadNode(
            name="source",
            source=data_source,
            column_info=column_info,
            config=self.config,
            start_time=start_time,
            end_time=end_time,
        )

        self.nodes.append(node)
        return node

    def build_aggregation_node(self, view, input_node: DAGNode) -> DAGNode:
        """Build aggregation node for Ray."""
        agg_specs = getattr(view, "aggregations", [])
        if not agg_specs:
            raise ValueError(f"No aggregations found for {view.name}")

        group_by_keys = view.entities
        timestamp_col = getattr(view.batch_source, "timestamp_field", "event_timestamp")

        node = RayAggregationNode(
            name="aggregation",
            aggregations=agg_specs,
            group_by_keys=group_by_keys,
            timestamp_col=timestamp_col,
            config=self.config,
        )
        node.add_input(input_node)

        self.nodes.append(node)
        return node

    def build_join_node(self, view, input_nodes):
        """Build the join node for combining multiple feature sources."""
        column_info = self.get_column_info(view)

        node = RayJoinNode(
            name="join",
            column_info=column_info,
            config=self.config,
            is_historical_retrieval=self.is_historical_retrieval,
        )
        for input_node in input_nodes:
            node.add_input(input_node)

        self.nodes.append(node)
        return node

    def build_filter_node(self, view, input_node):
        """Build the filter node for TTL and custom filtering."""
        ttl = getattr(view, "ttl", None)
        filter_condition = getattr(view, "filter", None)
        column_info = self.get_column_info(view)

        node = RayFilterNode(
            name="filter",
            column_info=column_info,
            config=self.config,
            ttl=ttl,
            filter_condition=filter_condition,
        )
        node.add_input(input_node)

        self.nodes.append(node)
        return node

    def build_dedup_node(self, view, input_node):
        """Build the deduplication node for removing duplicate records."""
        column_info = self.get_column_info(view)

        node = RayDedupNode(
            name="dedup",
            column_info=column_info,
            config=self.config,
        )
        node.add_input(input_node)

        self.nodes.append(node)
        return node

    def build_transformation_node(self, view, input_nodes):
        """Build the transformation node for user-defined transformations."""
        feature_transformation = getattr(view, "feature_transformation", None)
        udf = getattr(view, "udf", None)

        transformation = feature_transformation or udf
        if not transformation:
            raise ValueError(f"No feature transformation found for {view.name}")

        node = RayTransformationNode(
            name="transformation",
            transformation=transformation,
            config=self.config,
        )
        for input_node in input_nodes:
            node.add_input(input_node)

        self.nodes.append(node)
        return node

    def build_output_nodes(self, view, final_node):
        """Build the output node for writing processed features."""
        node = RayWriteNode(
            name="output",
            feature_view=view,
            inputs=[final_node],
        )

        self.nodes.append(node)
        return node

    def build_validation_node(self, view, input_node):
        """Build the validation node for feature validation."""
        # TODO: Implement validation logic
        logger.warning(
            "Feature validation is not yet implemented for Ray compute engine."
        )
        return input_node

    def _build(self, view, input_nodes: Optional[List[DAGNode]]) -> DAGNode:
        has_physical_source = (hasattr(view, "batch_source") and view.batch_source) or (
            hasattr(view, "source")
            and view.source
            and not isinstance(view.source, FeatureView)
        )

        is_derived_view = hasattr(view, "source_views") and view.source_views

        if has_physical_source and not is_derived_view:
            last_node = self.build_source_node(view)
            if self._should_transform(view):
                last_node = self.build_transformation_node(view, [last_node])
        elif input_nodes:
            if self._should_transform(view):
                last_node = self.build_transformation_node(view, input_nodes)
            else:
                last_node = self.build_join_node(view, input_nodes)
        else:
            raise ValueError(f"FeatureView {view.name} has no valid source or inputs")

        if last_node is None:
            raise ValueError(f"Failed to build processing node for {view.name}")

        last_node = self.build_filter_node(view, last_node)

        if self._should_aggregate(view):
            last_node = self.build_aggregation_node(view, last_node)
        elif self._should_dedupe(view):
            last_node = self.build_dedup_node(view, last_node)

        if self._should_validate(view):
            last_node = self.build_validation_node(view, last_node)

        return last_node

    def build(self) -> ExecutionPlan:
        """Build execution plan with support for derived feature views and sink_source writing."""
        if self.is_historical_retrieval and self._should_aggregate(self.feature_view):
            return self._build_aggregation_optimized_plan()

        if self.is_materialization:
            return self._build_materialization_plan()

        return super().build()

    def _build_materialization_plan(self) -> ExecutionPlan:
        """Build execution plan for materialization with intermediate sink writes."""
        logger.info(f"Building materialization plan for {self.feature_view.name}")

        # Step 1: Topo sort the FeatureViewNode DAG (Logical DAG)
        logical_nodes = self.feature_resolver.topological_sort(self.dag_root)
        logger.info(
            f"Logical nodes in topo order: {[node.view.name for node in logical_nodes]}"
        )

        # Step 2: For each FeatureView, build its corresponding execution DAGNode and write node
        # Build them in dependency order to ensure proper execution
        view_to_write_node: Dict[str, RayWriteNode] = {}

        for i, logical_node in enumerate(logical_nodes):
            view = logical_node.view
            logger.info(
                f"Building nodes for view {view.name} (step {i + 1}/{len(logical_nodes)})"
            )

            # For derived views, we need to ensure parent views are materialized first
            # So we create a processing chain that depends on parent write nodes
            parent_write_nodes = []
            processing_node: DAGNode
            if hasattr(view, "source_views") and view.source_views:
                # This is a derived view - collect parent write nodes as dependencies
                for parent in logical_node.inputs:
                    if parent.view.name in view_to_write_node:
                        parent_write_nodes.append(view_to_write_node[parent.view.name])

                if parent_write_nodes:
                    derived_read_node = RayDerivedReadNode(
                        name=f"{view.name}:derived_read",
                        feature_view=view,
                        parent_dependencies=cast(List[DAGNode], parent_write_nodes),
                        config=self.config,
                        column_info=self.get_column_info(view),
                        is_materialization=self.is_materialization,
                    )
                    self.nodes.append(derived_read_node)

                    # Then build the rest of the processing chain (filter, aggregate, etc.)
                    processing_node = self._build(view, [derived_read_node])
                else:
                    # Parent not yet built - this shouldn't happen in topo order
                    raise ValueError(f"Parent views for {view.name} not yet built")
            else:
                # Regular view - build normal processing chain
                processing_node = self._build(view, None)

            # Create a write node for this view
            write_node = RayWriteNode(
                name=f"{view.name}:write",
                feature_view=view,
                inputs=[processing_node],
            )

            view_to_write_node[view.name] = write_node
            logger.info(f"Created write node for {view.name}")

        # Step 3: The final write node is the one for the top-level feature view
        final_node = view_to_write_node[self.feature_view.name]

        # Step 4: Topo sort the final DAG from the output node (Physical DAG)
        sorted_nodes = topological_sort(final_node)

        # Step 5: Update self.nodes to include all nodes for the execution plan
        self.nodes = sorted_nodes

        # Step 6: Return sorted execution plan
        return ExecutionPlan(sorted_nodes)

    def _build_aggregation_optimized_plan(self) -> ExecutionPlan:
        """Build execution plan optimized for aggregation scenarios."""

        # 1. Read source data
        last_node = self.build_source_node(self.feature_view)

        # 2. Apply filters (TTL, custom filters) BEFORE aggregation
        last_node = self.build_filter_node(self.feature_view, last_node)

        # 3. Aggregate across all historical records
        last_node = self.build_aggregation_node(self.feature_view, last_node)

        # 4. Join with entity_df to get aggregated features for each entity
        last_node = self.build_join_node(self.feature_view, [last_node])

        # 5. Apply transformations to aggregated features
        if self._should_transform(self.feature_view):
            last_node = self.build_transformation_node(self.feature_view, [last_node])

        # 6. Output
        last_node = self.build_output_nodes(self.feature_view, last_node)

        return ExecutionPlan(self.nodes)



from typing import Any, Optional

from bigtree import Node
from colorama import Fore, Style

from feast import (
    BatchFeatureView,
    FeatureService,
    FeatureStore,
    FeatureView,
    OnDemandFeatureView,
    StreamFeatureView,
)
from feast.feast_object import FeastObject
from feast.permissions.action import ALL_ACTIONS
from feast.permissions.decision import DecisionEvaluator
from feast.permissions.permission import Permission
from feast.permissions.policy import Policy, RoleBasedPolicy
from feast.permissions.user import User


def print_permission_verbose_example():
    print("")
    print(
        f"{Style.BRIGHT + Fore.GREEN}The structure of the {Style.BRIGHT + Fore.WHITE}feast-permissions list --verbose {Style.BRIGHT + Fore.GREEN}command will be as in the following example:"
    )
    print("")
    print(f"{Style.DIM}For example: {Style.RESET_ALL}{Style.BRIGHT + Fore.GREEN}")
    print("")
    explanation_root_node = Node("permissions")
    explanation_permission_node = Node(
        "permission_1" + " " + str(["role names list"]),
        parent=explanation_root_node,
    )
    Node(
        FeatureView.__name__ + ": " + str(["feature view names"]),
        parent=explanation_permission_node,
    )
    Node(FeatureService.__name__ + ": none", parent=explanation_permission_node)
    Node("..", parent=explanation_permission_node)
    Node(
        "permission_2" + " " + str(["role names list"]),
        parent=explanation_root_node,
    )
    Node("..", parent=explanation_root_node)
    explanation_root_node.show()
    print(
        f"""
-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------{Style.RESET_ALL}
            """
    )


def handle_sd_verbose_permissions_command(
    feast_type: list[FeastObject],
    p: Permission,
    policy_node: Node,
    store: FeatureStore,
    tags_filter: Optional[dict[str, str]],
):
    saved_datasets = store.list_saved_datasets(tags=tags_filter)
    saved_datasets_names = set()
    for sd in saved_datasets:
        if p.match_resource(sd):
            saved_datasets_names.add(sd.name)
    if len(saved_datasets_names) > 0:
        Node(
            feast_type.__name__ + ": " + str(list(saved_datasets_names)),  # type: ignore[union-attr, attr-defined]
            parent=policy_node,
        )
    else:
        Node(feast_type.__name__ + ": none", parent=policy_node)  # type: ignore[union-attr, attr-defined]


def handle_vr_verbose_permissions_command(
    feast_type: list[FeastObject],
    p: Permission,
    policy_node: Node,
    store: FeatureStore,
    tags_filter: Optional[dict[str, str]],
):
    validation_references = store.list_validation_references(tags=tags_filter)
    validation_references_names = set()
    for vr in validation_references:
        if p.match_resource(vr):
            validation_references_names.add(vr.name)
    if len(validation_references_names) > 0:
        Node(
            feast_type.__name__ + ": " + str(list(validation_references_names)),  # type: ignore[union-attr, attr-defined]
            parent=policy_node,
        )
    else:
        Node(feast_type.__name__ + ": none", parent=policy_node)  # type: ignore[union-attr, attr-defined]


def handle_ds_verbose_permissions_command(
    feast_type: list[FeastObject],
    p: Permission,
    policy_node: Node,
    store: FeatureStore,
    tags_filter: Optional[dict[str, str]],
):
    data_sources = store.list_data_sources(tags=tags_filter)
    data_sources_names = set()
    for ds in data_sources:
        if p.match_resource(ds):
            data_sources_names.add(ds.name)
    if len(data_sources_names) > 0:
        Node(
            feast_type.__name__ + ": " + str(list(data_sources_names)),  # type: ignore[union-attr, attr-defined]
            parent=policy_node,
        )
    else:
        Node(feast_type.__name__ + ": none", parent=policy_node)  # type: ignore[union-attr, attr-defined]


def handle_fs_verbose_permissions_command(
    feast_type: list[FeastObject],
    p: Permission,
    policy_node: Node,
    store: FeatureStore,
    tags_filter: Optional[dict[str, str]],
):
    feature_services = store.list_feature_services(tags=tags_filter)
    feature_services_names = set()
    for fs in feature_services:
        if p.match_resource(fs):
            feature_services_names.add(fs.name)
    if len(feature_services_names) > 0:
        Node(
            feast_type.__name__ + ": " + str(list(feature_services_names)),  # type: ignore[union-attr, attr-defined]
            parent=policy_node,
        )
    else:
        Node(feast_type.__name__ + ": none", parent=policy_node)  # type: ignore[union-attr, attr-defined]


def handle_entity_verbose_permissions_command(
    feast_type: list[FeastObject],
    p: Permission,
    policy_node: Node,
    store: FeatureStore,
    tags_filter: Optional[dict[str, str]],
):
    entities = store.list_entities(tags=tags_filter)
    entities_names = set()
    for e in entities:
        if p.match_resource(e):
            entities_names.add(e.name)
    if len(entities_names) > 0:
        Node(feast_type.__name__ + ": " + str(list(entities_names)), parent=policy_node)  # type: ignore[union-attr, attr-defined]
    else:
        Node(feast_type.__name__ + ": none", parent=policy_node)  # type: ignore[union-attr, attr-defined]


def handle_fv_verbose_permissions_command(
    feast_type: list[FeastObject],
    p: Permission,
    policy_node: Node,
    store: FeatureStore,
    tags_filter: Optional[dict[str, str]],
):
    feature_views = []
    feature_views_names = set()
    if feast_type == FeatureView:
        feature_views = store.list_all_feature_views(tags=tags_filter)  # type: ignore[assignment]
    elif feast_type == OnDemandFeatureView:
        feature_views = store.list_on_demand_feature_views(
            tags=tags_filter  # type: ignore[assignment]
        )
    elif feast_type == BatchFeatureView:
        feature_views = store.list_batch_feature_views(tags=tags_filter)  # type: ignore[assignment]
    elif feast_type == StreamFeatureView:
        feature_views = store.list_stream_feature_views(
            tags=tags_filter  # type: ignore[assignment]
        )
    for fv in feature_views:
        if p.match_resource(fv):  # type: ignore[arg-type]
            feature_views_names.add(fv.name)
    if len(feature_views_names) > 0:
        Node(
            feast_type.__name__ + " " + str(list(feature_views_names)),  # type: ignore[union-attr, attr-defined]
            parent=policy_node,
        )
    else:
        Node(feast_type.__name__ + ": none", parent=policy_node)  # type: ignore[union-attr, attr-defined]


def handle_not_verbose_permissions_command(
    p: Permission, policy: Policy, table: list[Any]
):
    roles: set[str] = set()
    if isinstance(policy, RoleBasedPolicy):
        roles = set(policy.get_roles())
    table.append(
        [
            p.name,
            _to_multi_line([t.__name__ for t in p.types]),  # type: ignore[union-attr, attr-defined]
            _to_multi_line(p.name_patterns),
            _to_multi_line([a.value.upper() for a in p.actions]),
            _to_multi_line(sorted(roles)),
            _dict_to_multi_line(p.required_tags),
        ],
    )


def fetch_all_feast_objects(store: FeatureStore) -> list[FeastObject]:
    objects: list[FeastObject] = []
    objects.extend(store.list_entities())
    objects.extend(store.list_all_feature_views())  # type: ignore[arg-type]
    objects.extend(store.list_feature_services())
    objects.extend(store.list_data_sources())
    objects.extend(store.list_validation_references())
    objects.extend(store.list_saved_datasets())
    objects.extend(store.list_permissions())
    return objects


def handle_permissions_check_command(
    object: FeastObject, permissions: list[Permission], table: list[Any]
):
    for p in permissions:
        if p.match_resource(object):
            return
    table.append(
        [
            object.name,
            type(object).__name__,
        ]
    )


def handle_permissions_check_command_with_actions(
    object: FeastObject, permissions: list[Permission], table: list[Any]
):
    unmatched_actions = ALL_ACTIONS.copy()
    for p in permissions:
        if p.match_resource(object):
            for action in ALL_ACTIONS:
                if p.match_actions([action]) and action in unmatched_actions:
                    unmatched_actions.remove(action)

    if unmatched_actions:
        table.append(
            [
                object.name,
                type(object).__name__,
                _to_multi_line([a.value.upper() for a in unmatched_actions]),
            ]
        )


def fetch_all_permission_roles(permissions: list[Permission]) -> list[str]:
    all_roles = set()
    for p in permissions:
        if isinstance(p.policy, RoleBasedPolicy) and len(p.policy.get_roles()) > 0:
            all_roles.update(p.policy.get_roles())

    return sorted(all_roles)


def handler_list_all_permissions_roles(permissions: list[Permission], table: list[Any]):
    all_roles = fetch_all_permission_roles(permissions)
    for role in all_roles:
        table.append(
            [
                role,
            ]
        )


def handler_list_all_permissions_roles_verbose(
    objects: list[FeastObject], permissions: list[Permission], table: list[Any]
):
    all_roles = fetch_all_permission_roles(permissions)

    for role in all_roles:
        for o in objects:
            permitted_actions = ALL_ACTIONS.copy()
            for action in ALL_ACTIONS:
                # Following code is derived from enforcer.enforce_policy but has a different return type and does not raise FeastPermissionError
                matching_permissions = [
                    p
                    for p in permissions
                    if p.match_resource(o) and p.match_actions([action])
                ]

                if matching_permissions:
                    evaluator = DecisionEvaluator(
                        len(matching_permissions),
                    )
                    for p in matching_permissions:
                        permission_grant, permission_explanation = (
                            p.policy.validate_user(user=User(username="", roles=[role]))
                        )
                        evaluator.add_grant(
                            permission_grant,
                            f"Permission {p.name} denied access: {permission_explanation}",
                        )

                        if evaluator.is_decided():
                            grant, explanations = evaluator.grant()
                            if not grant:
                                permitted_actions.remove(action)
                            break
                else:
                    permitted_actions.remove(action)

            table.append(
                [
                    role,
                    o.name,
                    type(o).__name__,
                    _to_multi_line([a.value.upper() for a in permitted_actions]),
                ]
            )


def _to_multi_line(values: list[str]) -> str:
    if not values:
        return "-"
    return "\n".join(values)


def _dict_to_multi_line(values: dict[str, str]) -> str:
    if not values:
        return "-"
    return "\n".join([f"{key} : {value}" for key, value in values.items()])



from abc import ABC, abstractmethod
from typing import List, Sequence, Union

import pyarrow as pa

from feast import RepoConfig
from feast.batch_feature_view import BatchFeatureView
from feast.entity import Entity
from feast.feature_view import FeatureView
from feast.infra.common.materialization_job import (
    MaterializationJob,
    MaterializationTask,
)
from feast.infra.common.retrieval_task import HistoricalRetrievalTask
from feast.infra.compute_engines.dag.context import ExecutionContext
from feast.infra.offline_stores.offline_store import OfflineStore, RetrievalJob
from feast.infra.online_stores.online_store import OnlineStore
from feast.infra.registry.base_registry import BaseRegistry
from feast.on_demand_feature_view import OnDemandFeatureView
from feast.stream_feature_view import StreamFeatureView


class ComputeEngine(ABC):
    """
    The interface that Feast uses to control to compute system that handles materialization and get_historical_features.
    Each engine must implement:
        - materialize(): to generate and persist features
        - get_historical_features(): to perform historical retrieval of features
    Engines should use FeatureBuilder and DAGNode abstractions to build modular, pluggable workflows.
    """

    def __init__(
        self,
        *,
        repo_config: RepoConfig,
        offline_store: OfflineStore,
        online_store: OnlineStore,
        **kwargs,
    ):
        self.repo_config = repo_config
        self.offline_store = offline_store
        self.online_store = online_store

    @abstractmethod
    def update(
        self,
        project: str,
        views_to_delete: Sequence[
            Union[BatchFeatureView, StreamFeatureView, FeatureView]
        ],
        views_to_keep: Sequence[
            Union[BatchFeatureView, StreamFeatureView, FeatureView, OnDemandFeatureView]
        ],
        entities_to_delete: Sequence[Entity],
        entities_to_keep: Sequence[Entity],
    ):
        """
        Prepares cloud resources required for batch materialization for the specified set of Feast objects.

        Args:
            project: Feast project to which the objects belong.
            views_to_delete: Feature views whose corresponding infrastructure should be deleted.
            views_to_keep: Feature views whose corresponding infrastructure should not be deleted, and
                may need to be updated.
            entities_to_delete: Entities whose corresponding infrastructure should be deleted.
            entities_to_keep: Entities whose corresponding infrastructure should not be deleted, and
                may need to be updated.
        """
        pass

    @abstractmethod
    def teardown_infra(
        self,
        project: str,
        fvs: Sequence[Union[BatchFeatureView, StreamFeatureView, FeatureView]],
        entities: Sequence[Entity],
    ):
        """
        Tears down all cloud resources used by the materialization engine for the specified set of Feast objects.

        Args:
            project: Feast project to which the objects belong.
            fvs: Feature views whose corresponding infrastructure should be deleted.
            entities: Entities whose corresponding infrastructure should be deleted.
        """
        pass

    def materialize(
        self,
        registry: BaseRegistry,
        tasks: Union[MaterializationTask, List[MaterializationTask]],
        **kwargs,
    ) -> List[MaterializationJob]:
        if isinstance(tasks, MaterializationTask):
            tasks = [tasks]
        return [self._materialize_one(registry, task, **kwargs) for task in tasks]

    def _materialize_one(
        self,
        registry: BaseRegistry,
        task: MaterializationTask,
        **kwargs,
    ) -> MaterializationJob:
        raise NotImplementedError(
            "Materialization is not implemented for this compute engine."
        )

    def get_historical_features(
        self, registry: BaseRegistry, task: HistoricalRetrievalTask
    ) -> Union[RetrievalJob, pa.Table]:
        raise NotImplementedError

    def get_execution_context(
        self,
        registry: BaseRegistry,
        task: Union[MaterializationTask, HistoricalRetrievalTask],
    ) -> ExecutionContext:
        entity_defs = [
            registry.get_entity(name, task.project)
            for name in task.feature_view.entities
        ]
        entity_df = None
        if hasattr(task, "entity_df") and task.entity_df is not None:
            entity_df = task.entity_df

        return ExecutionContext(
            project=task.project,
            repo_config=self.repo_config,
            offline_store=self.offline_store,
            online_store=self.online_store,
            entity_defs=entity_defs,
            entity_df=entity_df,
        )

    def _get_feature_view_engine_config(
        self, feature_view: Union[BatchFeatureView, StreamFeatureView, FeatureView]
    ) -> dict:
        """
        Merge repo-level default batch engine config with runtime engine overrides defined in the feature view.

        Priority:
        1. Repo config (`self.repo_config.batch_engine_config`) - baseline
        2. FeatureView overrides (`batch_engine` for BatchFeatureView, `stream_engine` for StreamFeatureView`) - highest priority

        Args:
            feature_view: A BatchFeatureView or StreamFeatureView.

        Returns:
            dict: The merged engine configuration.
        """
        default_conf = self.repo_config.batch_engine_config or {}

        runtime_conf = None
        if isinstance(feature_view, BatchFeatureView):
            runtime_conf = feature_view.batch_engine
        elif isinstance(feature_view, StreamFeatureView):
            runtime_conf = feature_view.stream_engine

        if runtime_conf is not None and not isinstance(runtime_conf, dict):
            raise TypeError(
                f"Engine config for {feature_view.name} must be a dict, got {type(runtime_conf)}."
            )

        return {**default_conf, **runtime_conf} if runtime_conf else dict(default_conf)



import logging

from fastapi import APIRouter, Depends, Query

from feast.api.registry.rest.codegen_utils import render_entity_code
from feast.api.registry.rest.rest_utils import (
    aggregate_across_projects,
    create_grpc_pagination_params,
    create_grpc_sorting_params,
    get_object_relationships,
    get_pagination_params,
    get_relationships_for_objects,
    get_sorting_params,
    grpc_call,
)
from feast.protos.feast.registry import RegistryServer_pb2

logger = logging.getLogger(__name__)


def get_entity_router(grpc_handler) -> APIRouter:
    router = APIRouter()

    @router.get("/entities")
    def list_entities(
        project: str = Query(...),
        allow_cache: bool = Query(default=True),
        include_relationships: bool = Query(
            False, description="Include relationships for each entity"
        ),
        pagination_params: dict = Depends(get_pagination_params),
        sorting_params: dict = Depends(get_sorting_params),
    ):
        req = RegistryServer_pb2.ListEntitiesRequest(
            project=project,
            allow_cache=allow_cache,
            pagination=create_grpc_pagination_params(pagination_params),
            sorting=create_grpc_sorting_params(sorting_params),
        )
        response = grpc_call(grpc_handler.ListEntities, req)
        entities = response.get("entities", [])

        result = {
            "entities": entities,
            "pagination": response.get("pagination", {}),
        }

        if include_relationships:
            relationships = get_relationships_for_objects(
                grpc_handler, entities, "entity", project, allow_cache
            )
            result["relationships"] = relationships

        return result

    @router.get("/entities/all")
    def list_all_entities(
        allow_cache: bool = Query(default=True),
        page: int = Query(1, ge=1),
        limit: int = Query(50, ge=1, le=100),
        sort_by: str = Query(None),
        sort_order: str = Query("asc"),
        include_relationships: bool = Query(
            False, description="Include relationships for each entity"
        ),
    ):
        return aggregate_across_projects(
            grpc_handler=grpc_handler,
            list_method=grpc_handler.ListEntities,
            request_cls=RegistryServer_pb2.ListEntitiesRequest,
            response_key="entities",
            object_type="entity",
            allow_cache=allow_cache,
            page=page,
            limit=limit,
            sort_by=sort_by,
            sort_order=sort_order,
            include_relationships=include_relationships,
        )

    @router.get("/entities/{name}")
    def get_entity(
        name: str,
        project: str = Query(...),
        include_relationships: bool = Query(
            False, description="Include relationships for this entity"
        ),
        allow_cache: bool = Query(default=True),
    ):
        req = RegistryServer_pb2.GetEntityRequest(
            name=name,
            project=project,
            allow_cache=allow_cache,
        )
        entity = grpc_call(grpc_handler.GetEntity, req)

        result = entity

        relationships = get_object_relationships(
            grpc_handler, "entity", name, project, allow_cache
        )
        ds_list_req = RegistryServer_pb2.ListDataSourcesRequest(
            project=project,
            allow_cache=allow_cache,
        )
        ds_list_resp = grpc_call(grpc_handler.ListDataSources, ds_list_req)
        ds_map = {ds["name"]: ds for ds in ds_list_resp.get("dataSources", [])}
        data_source_objs = []
        seen_ds_names = set()
        for rel in relationships:
            if rel.get("target", {}).get("type") == "dataSource":
                ds_name = rel["target"].get("name")
                if ds_name and ds_name not in seen_ds_names:
                    ds_obj = ds_map.get(ds_name)
                    if ds_obj:
                        data_source_objs.append(ds_obj)
                        seen_ds_names.add(ds_name)
        result["dataSources"] = data_source_objs

        if include_relationships:
            result["relationships"] = relationships

        if result:
            spec = result.get("spec", result)
            name = spec.get("name") or result.get("name") or "default_entity"
            join_keys = spec.get("joinKeys") or (
                [spec["joinKey"]] if "joinKey" in spec else []
            )

            context = {
                "name": name,
                "join_keys": join_keys,
                "description": spec.get("description", ""),
                "tags": spec.get("tags", {}),
            }
            result["featureDefinition"] = render_entity_code(context)
        return result

    return router



import logging
from typing import Any, Callable, Dict, List, Optional

from fastapi import HTTPException, Query
from google.protobuf.json_format import MessageToDict

from feast.errors import (
    FeastObjectNotFoundException,
    FeastPermissionError,
    PushSourceNotFoundException,
)
from feast.protos.feast.registry import RegistryServer_pb2

logger = logging.getLogger(__name__)


MATCH_SCORE_DEFAULT_THRESHOLD = 0.75
MATCH_SCORE_NAME = 100
MATCH_SCORE_DESCRIPTION = 80
MATCH_SCORE_TAGS = 60
MATCH_SCORE_PARTIAL = 40


def grpc_call(handler_fn, request):
    """
    Wrapper to invoke gRPC method with context=None and handle common errors.
    """
    try:
        response = handler_fn(request, context=None)
        return MessageToDict(response)
    except (
        FeastObjectNotFoundException,
        FeastPermissionError,
        PushSourceNotFoundException,
    ):
        raise
    except Exception as e:
        raise e


def get_object_relationships(
    grpc_handler,
    object_type: str,
    object_name: str,
    project: str,
    allow_cache: bool = True,
) -> List:
    """
    Get relationships for a specific object.

    Args:
        grpc_handler: The gRPC handler to use for calls
        object_type: Type of object (dataSource, entity, featureView, featureService)
        object_name: Name of the object
        project: Project name
        allow_cache: Whether to allow cached data

    Returns:
        List containing relationships for the object (both direct and indirect)
    """
    try:
        req = RegistryServer_pb2.GetObjectRelationshipsRequest(
            project=project,
            object_type=object_type,
            object_name=object_name,
            include_indirect=True,
            allow_cache=allow_cache,
        )
        response = grpc_call(grpc_handler.GetObjectRelationships, req)
        return response.get("relationships", [])
    except Exception:
        # If relationships can't be retrieved, return empty list rather than failing
        return []


def get_relationships_for_objects(
    grpc_handler,
    objects: List[Dict],
    object_type: str,
    project: str,
    allow_cache: bool = True,
) -> Dict[str, List]:
    """
    Get relationships for multiple objects efficiently.

    Args:
        grpc_handler: The gRPC handler to use for calls
        objects: List of objects to get relationships for
        object_type: Type of objects (dataSource, entity, featureView, featureService, feature)
        project: Project name
        allow_cache: Whether to allow cached data

    Returns:
        Dictionary mapping object names to their relationships (both direct and indirect)
    """
    relationships_map = {}

    for obj in objects:
        obj_name = None
        if object_type == "feature":
            obj_name = obj.get("name")
        elif isinstance(obj, dict):
            obj_name = (
                obj.get("name")
                or obj.get("spec", {}).get("name")
                or obj.get("meta", {}).get("name")
            )

        if obj_name:
            rels = get_object_relationships(
                grpc_handler,
                object_type,
                obj_name,
                project,
                allow_cache,
            )
            relationships_map[obj_name] = rels

    return relationships_map


def aggregate_across_projects(
    grpc_handler,
    list_method: Callable,
    request_cls: Callable,
    response_key: str,
    object_type: str,
    allow_cache: bool = True,
    page: int = 1,
    limit: int = 50,
    sort_by: Optional[str] = None,
    sort_order: str = "asc",
    include_relationships: bool = False,
) -> Dict:
    """
    Fetches and aggregates objects across all projects, adds project field, handles relationships, and paginates/sorts.
    """
    projects_resp = grpc_call(
        grpc_handler.ListProjects,
        RegistryServer_pb2.ListProjectsRequest(allow_cache=allow_cache),
    )
    projects = projects_resp.get("projects", [])
    all_objects = []
    relationships_map = {}

    for project in projects:
        project_name = project["spec"]["name"]
        req = request_cls(
            project=project_name,
            allow_cache=allow_cache,
        )
        response = grpc_call(list_method, req)
        objects = response.get(response_key, [])
        for obj in objects:
            obj["project"] = project_name
        all_objects.extend(objects)
        if include_relationships:
            rels = get_relationships_for_objects(
                grpc_handler, objects, object_type, project_name, allow_cache
            )
            relationships_map.update(rels)

    paged_objects, pagination = paginate_and_sort(
        all_objects, page, limit, sort_by, sort_order
    )
    result = {
        response_key: paged_objects,
        "pagination": pagination,
    }
    if include_relationships:
        result["relationships"] = relationships_map
    return result


def get_all_feature_views(feature_views_response: dict) -> list[dict]:
    """
    Get all feature views from a feature views response, regardless of type.
    This is future-proof and will handle any kind of feature view keys.
    """
    result = []
    for key, value in feature_views_response.items():
        if isinstance(value, list):
            result.extend(value)
        else:
            result.append(value)
    return result


def parse_tags(tags: List[str] = Query(default=[])) -> Dict[str, str]:
    """
    Parses query strings like ?tags=key1:value1&tags=key2:value2 into a dict.
    """
    parsed_tags = {}
    for tag in tags:
        if ":" not in tag:
            continue
        key, value = tag.split(":", 1)
        parsed_tags[key] = value
    return parsed_tags


def get_pagination_params(
    page: Optional[int] = Query(None, ge=1),
    limit: Optional[int] = Query(None, ge=1, le=100),
) -> dict:
    return {
        "page": page or 0,
        "limit": limit or 0,
    }


def get_sorting_params(
    sort_by: Optional[str] = Query(None),
    sort_order: Optional[str] = Query(None, pattern="^(asc|desc)$"),
) -> dict:
    return {
        "sort_by": sort_by or "",
        "sort_order": sort_order or "asc",
    }


def validate_or_set_default_sorting_params(
    sort_by_options: List[str] = [],
    default_sort_by_option: str = "",
    default_sort_order: str = "asc",
) -> Callable:
    """
    Factory function to create a FastAPI dependency for validating sorting parameters.

    Args:
        sort_by_options: List of valid sort_by field names. If empty, no validation is performed
        default_sort_by_option: Default sort_by value if not provided
        default_sort_order: Default sort_order value if not provided (asc/desc)

    Returns:
        Callable that can be used as FastAPI dependency for sorting validation

    Example usage:
        # Create a custom sorting validator for specific fields
        custom_sorting = validate_or_set_default_sorting_params(
            sort_by_options=["name", "created_at", "updated_at"],
            default_sort_by_option="name",
            default_sort_order="asc"
        )

        # Use in FastAPI route
        @router.get("/items")
        def get_items(sorting_params: dict = Depends(custom_sorting)):
            sort_by = sorting_params["sort_by"]
            sort_order = sorting_params["sort_order"]
            # Use sort_by and sort_order for your logic
    """

    def set_input_or_default(
        sort_by: Optional[str] = Query(None), sort_order: Optional[str] = Query(None)
    ) -> dict:
        sorting_params = {}

        # If no sort options are configured, return defaults without validation
        if not sort_by_options:
            return {
                "sort_by": default_sort_by_option,
                "sort_order": sort_order if sort_order else default_sort_order,
            }

        # Validate and set sort_by parameter
        if sort_by:
            if sort_by in sort_by_options:
                sorting_params["sort_by"] = sort_by
            else:
                raise HTTPException(
                    status_code=400,
                    detail=f"Invalid sort_by parameter: '{sort_by}'. Valid options are: {sort_by_options}",
                )
        else:
            # Use default if not provided
            sorting_params["sort_by"] = default_sort_by_option

        # Validate and set sort_order parameter
        if sort_order:
            if sort_order in ["asc", "desc"]:
                sorting_params["sort_order"] = sort_order
            else:
                raise HTTPException(
                    status_code=400,
                    detail=f"Invalid sort_order parameter: '{sort_order}'. Valid options are: ['asc', 'desc']",
                )
        else:
            # Use default if not provided
            sorting_params["sort_order"] = default_sort_order

        return sorting_params

    return set_input_or_default


def validate_or_set_default_pagination_params(
    default_page: int = 1,
    default_limit: int = 50,
    min_page: int = 1,
    min_limit: int = 1,
    max_limit: int = 100,
) -> Callable:
    """
    Factory function to create a FastAPI dependency for validating pagination parameters.

    Args:
        default_page: Default page number if not provided
        default_limit: Default limit if not provided
        min_page: Minimum allowed page number
        min_limit: Minimum allowed limit
        max_limit: Maximum allowed limit

    Returns:
        Callable that can be used as FastAPI dependency for pagination validation

    Example usage:
        # Create a custom pagination validator
        custom_pagination = validate_or_set_default_pagination_params(
            default_page=1,
            default_limit=25,
            max_limit=200
        )

        # Use in FastAPI route
        @router.get("/items")
        def get_items(pagination_params: dict = Depends(custom_pagination)):
            page = pagination_params["page"]
            limit = pagination_params["limit"]
            # Use page and limit for your logic
    """

    def set_input_or_default(
        page: Optional[int] = Query(None), limit: Optional[int] = Query(None)
    ) -> dict:
        pagination_params = {}

        # Validate and set page parameter
        if page is not None:
            if page < min_page:
                raise HTTPException(
                    status_code=400,
                    detail=f"Invalid page parameter: '{page}'. Must be greater than or equal to {min_page}",
                )
            pagination_params["page"] = page
        else:
            pagination_params["page"] = default_page

        # Validate and set limit parameter
        if limit is not None:
            if limit < min_limit:
                raise HTTPException(
                    status_code=400,
                    detail=f"Invalid limit parameter: '{limit}'. Must be greater than or equal to {min_limit}",
                )
            if limit > max_limit:
                raise HTTPException(
                    status_code=400,
                    detail=f"Invalid limit parameter: '{limit}'. Must be less than or equal to {max_limit}",
                )
            pagination_params["limit"] = limit
        else:
            pagination_params["limit"] = default_limit

        return pagination_params

    return set_input_or_default


def create_grpc_pagination_params(
    pagination_params: dict,
) -> RegistryServer_pb2.PaginationParams:
    return RegistryServer_pb2.PaginationParams(
        page=pagination_params.get("page", 0),
        limit=pagination_params.get("limit", 0),
    )


def create_grpc_sorting_params(
    sorting_params: dict,
) -> RegistryServer_pb2.SortingParams:
    return RegistryServer_pb2.SortingParams(
        sort_by=sorting_params.get("sort_by", ""),
        sort_order=sorting_params.get("sort_order", "asc"),
    )


def paginate_and_sort(
    items: list,
    page: int,
    limit: int,
    sort_by: Optional[str] = None,
    sort_order: str = "asc",
):
    if sort_by:
        items = sorted(
            items, key=lambda x: x.get(sort_by, ""), reverse=(sort_order == "desc")
        )
    total = len(items)
    start = (page - 1) * limit
    end = start + limit
    paged_items = items[start:end]
    pagination = {}
    if page:
        pagination["page"] = page
    if limit:
        pagination["limit"] = limit
    if total:
        pagination["totalCount"] = total
    total_pages = (total + limit - 1) // limit
    if total_pages:
        pagination["totalPages"] = total_pages
    if end < total:
        pagination["hasNext"] = True
    if start > 0:
        pagination["hasPrevious"] = True
    return paged_items, pagination


def get_all_project_resources(
    grpc_handler,
    project: str,
    allow_cache: bool,
    tags: Optional[Dict[str, str]] = None,
    pagination_params: Optional[dict] = None,
    sorting_params: Optional[dict] = None,
) -> tuple[Dict[str, Any], Dict[str, Any], List[str]]:
    """
    Helper function to get all resources for a project with optional sorting and pagination
    Returns a dictionary with resource types as keys and lists of resources as values
    Also includes pagination metadata when pagination_params are provided
    """

    resources: Dict[str, Any] = {
        "entities": [],
        "dataSources": [],
        "featureViews": [],
        "featureServices": [],
        "savedDatasets": [],
        "features": [],
        "pagination": {},
        "errors": [],
    }
    pagination: dict = {}
    errors = []

    try:
        # Get entities
        resources["entities"], pagination["entities"], err_msg = list_entities(
            grpc_handler=grpc_handler,
            project=project,
            allow_cache=allow_cache,
            tags=tags,
            pagination_params=pagination_params,
            sorting_params=sorting_params,
        )
        if err_msg:
            errors.append(err_msg)

        # Get data sources
        resources["dataSources"], pagination["dataSources"], err_msg = (
            list_data_sources(
                grpc_handler=grpc_handler,
                project=project,
                allow_cache=allow_cache,
                tags=tags,
                pagination_params=pagination_params,
                sorting_params=sorting_params,
            )
        )
        if err_msg:
            errors.append(err_msg)

        # Get feature views
        resources["featureViews"], pagination["featureViews"], err_msg = (
            list_feature_views(
                grpc_handler=grpc_handler,
                project=project,
                allow_cache=allow_cache,
                tags=tags,
                pagination_params=pagination_params,
                sorting_params=sorting_params,
            )
        )
        if err_msg:
            errors.append(err_msg)

        # Get feature services
        resources["featureServices"], pagination["featureServices"], err_msg = (
            list_feature_services(
                grpc_handler=grpc_handler,
                project=project,
                allow_cache=allow_cache,
                tags=tags,
                pagination_params=pagination_params,
                sorting_params=sorting_params,
            )
        )
        if err_msg:
            errors.append(err_msg)

        # Get saved datasets
        resources["savedDatasets"], pagination["savedDatasets"], err_msg = (
            list_saved_datasets(
                grpc_handler=grpc_handler,
                project=project,
                allow_cache=allow_cache,
                tags=tags,
                pagination_params=pagination_params,
                sorting_params=sorting_params,
            )
        )
        if err_msg:
            errors.append(err_msg)

        # Get features
        resources["features"], pagination["features"], err_msg = list_features(
            grpc_handler=grpc_handler,
            project=project,
            allow_cache=allow_cache,
            pagination_params=pagination_params,
            sorting_params=sorting_params,
        )
        if err_msg:
            errors.append(err_msg)

    except Exception as e:
        err_msg = f"Error getting resources for project '{project}'"
        errors.append(err_msg)
        logger.error(f"{err_msg}: {e}")
    finally:
        return resources, pagination, errors


def filter_search_results_and_match_score(
    results: List[Dict], query: str
) -> List[Dict]:
    """Filter search results based on query string"""
    if not query:
        return results

    query_lower = query.lower()
    filtered_results = []

    for result in results:
        # Search in name
        if query_lower in result.get("name", "").lower():
            result["match_score"] = MATCH_SCORE_NAME
            filtered_results.append(result)
            continue

        # Search in description
        if query_lower in result.get("description", "").lower():
            result["match_score"] = MATCH_SCORE_DESCRIPTION
            filtered_results.append(result)
            continue

        # Search in tags
        tags = result.get("tags", {})
        tag_match = False
        for key, value in tags.items():
            if query_lower in key.lower() or query_lower in str(value).lower():
                tag_match = True
                break

        if tag_match:
            result["match_score"] = MATCH_SCORE_TAGS
            filtered_results.append(result)
            continue

        # Partial name match (fuzzy search)
        fuzzy_match_score = fuzzy_match(query_lower, result.get("name", "").lower())
        if fuzzy_match_score >= MATCH_SCORE_DEFAULT_THRESHOLD:
            result["match_score"] = fuzzy_match_score * 100
            filtered_results.append(result)

    return filtered_results


def fuzzy_match(query: str, text: str) -> float:
    """Simple fuzzy matching using character overlap"""
    if not query or not text:
        return 0.0

    query_chars = set(query)
    text_chars = set(text)

    overlap = len(query_chars.intersection(text_chars))
    similarity = overlap / len(query_chars.union(text_chars))

    return similarity


def list_entities(
    grpc_handler,
    project: str,
    allow_cache: bool,
    tags: Optional[Dict[str, str]] = None,
    pagination_params: Optional[dict] = None,
    sorting_params: Optional[dict] = None,
) -> tuple[List[Dict[str, Any]], Dict[str, Any], str]:
    """
    Search entities in a project with optional sorting and pagination
    """
    entities = []
    pagination = {}
    err_msg = ""

    grpc_pagination = None
    grpc_sorting = None

    if pagination_params:
        grpc_pagination = create_grpc_pagination_params(pagination_params)
    if sorting_params:
        grpc_sorting = create_grpc_sorting_params(sorting_params)

    try:
        entities_req = RegistryServer_pb2.ListEntitiesRequest(
            project=project,
            allow_cache=allow_cache,
            pagination=grpc_pagination,
            sorting=grpc_sorting,
            tags=tags,
        )
        entities_response = grpc_call(grpc_handler.ListEntities, entities_req)
        entities, pagination = (
            entities_response.get("entities", []),
            entities_response.get("pagination", {}),
        )
    except Exception as e:
        err_msg = f"Error searching entities in project '{project}'"
        logger.error(f"{err_msg}: {e}")
    finally:
        return entities, pagination, err_msg


def list_feature_views(
    grpc_handler,
    project: str,
    allow_cache: bool,
    tags: Optional[Dict[str, str]] = None,
    pagination_params: Optional[dict] = None,
    sorting_params: Optional[dict] = None,
) -> tuple[List[Dict[str, Any]], Dict[str, Any], str]:
    """
    Search feature views in a project with optional sorting and pagination
    """
    feature_views = []
    pagination = {}
    err_msg = ""

    grpc_pagination = None
    grpc_sorting = None

    if pagination_params:
        grpc_pagination = create_grpc_pagination_params(pagination_params)
    if sorting_params:
        grpc_sorting = create_grpc_sorting_params(sorting_params)

    try:
        feature_views_req = RegistryServer_pb2.ListAllFeatureViewsRequest(
            project=project,
            allow_cache=allow_cache,
            pagination=grpc_pagination,
            sorting=grpc_sorting,
            tags=tags,
        )
        feature_views_response = grpc_call(
            grpc_handler.ListAllFeatureViews, feature_views_req
        )
        all_feature_views = get_all_feature_views(feature_views_response)
        feature_views, pagination = (
            all_feature_views,
            feature_views_response.get("pagination", {}),
        )
    except Exception as e:
        err_msg = f"Error searching feature views in project '{project}'"
        logger.error(f"{err_msg}: {e}")
    finally:
        return feature_views, pagination, err_msg


def list_feature_services(
    grpc_handler,
    project: str,
    allow_cache: bool,
    tags: Optional[Dict[str, str]] = None,
    pagination_params: Optional[dict] = None,
    sorting_params: Optional[dict] = None,
) -> tuple[List[Dict[str, Any]], Dict[str, Any], str]:
    """
    Search feature services in a project with optional sorting and pagination
    """
    feature_services = []
    pagination = {}
    err_msg = ""

    grpc_pagination = None
    grpc_sorting = None

    if pagination_params:
        grpc_pagination = create_grpc_pagination_params(pagination_params)
    if sorting_params:
        grpc_sorting = create_grpc_sorting_params(sorting_params)

    try:
        feature_services_req = RegistryServer_pb2.ListFeatureServicesRequest(
            project=project,
            allow_cache=allow_cache,
            pagination=grpc_pagination,
            sorting=grpc_sorting,
            tags=tags,
        )
        feature_services_response = grpc_call(
            grpc_handler.ListFeatureServices, feature_services_req
        )
        feature_services, pagination = (
            feature_services_response.get("featureServices", []),
            feature_services_response.get("pagination", {}),
        )
    except Exception as e:
        err_msg = f"Error searching feature services in project '{project}'"
        logger.error(f"{err_msg}: {e}")
    finally:
        return feature_services, pagination, err_msg


def list_data_sources(
    grpc_handler,
    project: str,
    allow_cache: bool,
    tags: Optional[Dict[str, str]] = None,
    pagination_params: Optional[dict] = None,
    sorting_params: Optional[dict] = None,
) -> tuple[List[Dict[str, Any]], Dict[str, Any], str]:
    """
    Search data sources in a project with optional sorting and pagination
    """
    data_sources = []
    pagination = {}
    err_msg = ""

    grpc_pagination = None
    grpc_sorting = None

    if pagination_params:
        grpc_pagination = create_grpc_pagination_params(pagination_params)
    if sorting_params:
        grpc_sorting = create_grpc_sorting_params(sorting_params)

    try:
        data_sources_req = RegistryServer_pb2.ListDataSourcesRequest(
            project=project,
            allow_cache=allow_cache,
            pagination=grpc_pagination,
            sorting=grpc_sorting,
            tags=tags,
        )
        data_sources_response = grpc_call(
            grpc_handler.ListDataSources, data_sources_req
        )
        data_sources, pagination = (
            data_sources_response.get("dataSources", []),
            data_sources_response.get("pagination", {}),
        )
    except Exception as e:
        err_msg = f"Error searching data sources in project '{project}'"
        logger.error(f"{err_msg}: {e}")
    finally:
        return data_sources, pagination, err_msg


def list_saved_datasets(
    grpc_handler,
    project: str,
    allow_cache: bool,
    tags: Optional[Dict[str, str]] = None,
    pagination_params: Optional[dict] = None,
    sorting_params: Optional[dict] = None,
) -> tuple[List[Dict[str, Any]], Dict[str, Any], str]:
    """
    Search saved datasets in a project with optional sorting and pagination
    """
    saved_datasets = []
    pagination = {}
    err_msg = ""

    grpc_pagination = None
    grpc_sorting = None

    if pagination_params:
        grpc_pagination = create_grpc_pagination_params(pagination_params)
    if sorting_params:
        grpc_sorting = create_grpc_sorting_params(sorting_params)

    try:
        saved_datasets_req = RegistryServer_pb2.ListSavedDatasetsRequest(
            project=project,
            allow_cache=allow_cache,
            pagination=grpc_pagination,
            sorting=grpc_sorting,
            tags=tags,
        )
        saved_datasets_response = grpc_call(
            grpc_handler.ListSavedDatasets, saved_datasets_req
        )
        saved_datasets, pagination = (
            saved_datasets_response.get("savedDatasets", []),
            saved_datasets_response.get("pagination", {}),
        )
    except Exception as e:
        err_msg = f"Error searching saved datasets in project '{project}'"
        logger.error(f"{err_msg}: {e}")
    finally:
        return saved_datasets, pagination, err_msg


def list_features(
    grpc_handler,
    project: str,
    allow_cache: bool,
    pagination_params: Optional[dict] = None,
    sorting_params: Optional[dict] = None,
) -> tuple[List[Dict[str, Any]], Dict[str, Any], str]:
    """
    Search features in a project with optional sorting and pagination
    """
    features = []
    pagination = {}
    err_msg = ""

    grpc_pagination = None
    grpc_sorting = None

    if pagination_params:
        grpc_pagination = create_grpc_pagination_params(pagination_params)
    if sorting_params:
        grpc_sorting = create_grpc_sorting_params(sorting_params)

    try:
        features_req = RegistryServer_pb2.ListFeaturesRequest(
            project=project,
            allow_cache=allow_cache,
            pagination=grpc_pagination,
            sorting=grpc_sorting,
        )
        features_response = grpc_call(grpc_handler.ListFeatures, features_req)
        features, pagination = (
            features_response.get("features", []),
            features_response.get("pagination", {}),
        )
    except Exception as e:
        err_msg = f"Error searching features in project '{project}'"
        logger.error(f"{err_msg}: {e}")
    finally:
        return features, pagination, err_msg


def list_all_projects(
    grpc_handler,
    allow_cache: bool,
    tags: Optional[Dict[str, str]] = None,
    pagination_params: Optional[dict] = None,
    sorting_params: Optional[dict] = None,
) -> tuple[List[Dict[str, Any]], Dict[str, Any], str]:
    """
    Search all projects with optional sorting and pagination
    """
    projects = []
    pagination = {}
    err_msg = ""

    grpc_pagination = None
    grpc_sorting = None

    if pagination_params:
        grpc_pagination = create_grpc_pagination_params(pagination_params)
    if sorting_params:
        grpc_sorting = create_grpc_sorting_params(sorting_params)

    try:
        projects_req = RegistryServer_pb2.ListProjectsRequest(
            allow_cache=allow_cache,
            pagination=grpc_pagination,
            sorting=grpc_sorting,
            tags=tags,
        )
        projects_response = grpc_call(grpc_handler.ListProjects, projects_req)
        projects, pagination = (
            projects_response.get("projects", []),
            projects_response.get("pagination", {}),
        )
    except Exception as e:
        err_msg = "Error searching all projects"
        logger.error(f"{err_msg}: {e}")
    finally:
        return projects, pagination, err_msg



import os
from datetime import datetime
from pathlib import Path
from typing import Any, Callable, List, Optional, Union

import ibis
import pandas as pd
import pyarrow
from ibis.expr.types import Table
from pydantic import StrictStr

from feast.data_format import DeltaFormat, ParquetFormat
from feast.data_source import DataSource
from feast.errors import SavedDatasetLocationAlreadyExists
from feast.feature_logging import LoggingConfig, LoggingSource
from feast.feature_view import FeatureView
from feast.infra.offline_stores.file_source import FileSource
from feast.infra.offline_stores.ibis import (
    get_historical_features_ibis,
    offline_write_batch_ibis,
    pull_all_from_table_or_query_ibis,
    pull_latest_from_table_or_query_ibis,
    write_logged_features_ibis,
)
from feast.infra.offline_stores.offline_store import OfflineStore, RetrievalJob
from feast.infra.registry.base_registry import BaseRegistry
from feast.repo_config import FeastConfigBaseModel, RepoConfig


def _read_data_source(data_source: DataSource, repo_path: str) -> Table:
    assert isinstance(data_source, FileSource)

    if isinstance(data_source.file_format, ParquetFormat):
        return ibis.read_parquet(data_source.path)
    elif isinstance(data_source.file_format, DeltaFormat):
        storage_options = {
            "AWS_ENDPOINT_URL": data_source.s3_endpoint_override,
        }

        return ibis.read_delta(data_source.path, storage_options=storage_options)


def _write_data_source(
    table: Table,
    data_source: DataSource,
    repo_path: str,
    mode: str = "append",
    allow_overwrite: bool = False,
):
    assert isinstance(data_source, FileSource)

    file_options = data_source.file_options

    absolute_path = FileSource.get_uri_for_file_path(
        repo_path=repo_path, uri=file_options.uri
    )

    if (
        mode == "overwrite"
        and not allow_overwrite
        and os.path.exists(str(absolute_path))
    ):
        raise SavedDatasetLocationAlreadyExists(location=file_options.uri)

    if isinstance(data_source.file_format, ParquetFormat):
        if mode == "overwrite":
            table = table.to_pyarrow()

            filesystem, path = FileSource.create_filesystem_and_path(
                str(absolute_path),
                file_options.s3_endpoint_override,
            )

            if path.endswith(".parquet"):
                pyarrow.parquet.write_table(table, where=path, filesystem=filesystem)
            else:
                # otherwise assume destination is directory
                pyarrow.parquet.write_to_dataset(
                    table, root_path=path, filesystem=filesystem
                )
        elif mode == "append":
            table = table.to_pyarrow()
            prev_table = ibis.read_parquet(file_options.uri).to_pyarrow()
            if table.schema != prev_table.schema:
                table = table.cast(prev_table.schema)
            new_table = pyarrow.concat_tables([table, prev_table])
            ibis.memtable(new_table).to_parquet(file_options.uri)
    elif isinstance(data_source.file_format, DeltaFormat):
        storage_options = {
            "AWS_ENDPOINT_URL": str(data_source.s3_endpoint_override),
        }

        if mode == "append":
            from deltalake import DeltaTable

            prev_schema = (
                DeltaTable(file_options.uri, storage_options=storage_options)
                .schema()
                .to_pyarrow()
            )
            table = table.cast(ibis.Schema.from_pyarrow(prev_schema))
            write_mode = "append"
        elif mode == "overwrite":
            write_mode = (
                "overwrite"
                if allow_overwrite and os.path.exists(file_options.uri)
                else "error"
            )

        table.to_delta(
            file_options.uri, mode=write_mode, storage_options=storage_options
        )


class DuckDBOfflineStoreConfig(FeastConfigBaseModel):
    type: StrictStr = "duckdb"
    # """ Offline store type selector"""

    staging_location: Optional[str] = None

    staging_location_endpoint_override: Optional[str] = None


class DuckDBOfflineStore(OfflineStore):
    @staticmethod
    def pull_latest_from_table_or_query(
        config: RepoConfig,
        data_source: DataSource,
        join_key_columns: List[str],
        feature_name_columns: List[str],
        timestamp_field: str,
        created_timestamp_column: Optional[str],
        start_date: datetime,
        end_date: datetime,
    ) -> RetrievalJob:
        return pull_latest_from_table_or_query_ibis(
            config=config,
            data_source=data_source,
            join_key_columns=join_key_columns,
            feature_name_columns=feature_name_columns,
            timestamp_field=timestamp_field,
            created_timestamp_column=created_timestamp_column,
            start_date=start_date,
            end_date=end_date,
            data_source_reader=_read_data_source,
            data_source_writer=_write_data_source,
            staging_location=config.offline_store.staging_location,
            staging_location_endpoint_override=config.offline_store.staging_location_endpoint_override,
        )

    @staticmethod
    def get_historical_features(
        config: RepoConfig,
        feature_views: List[FeatureView],
        feature_refs: List[str],
        entity_df: Union[pd.DataFrame, str],
        registry: BaseRegistry,
        project: str,
        full_feature_names: bool = False,
    ) -> RetrievalJob:
        return get_historical_features_ibis(
            config=config,
            feature_views=feature_views,
            feature_refs=feature_refs,
            entity_df=entity_df,
            registry=registry,
            project=project,
            full_feature_names=full_feature_names,
            data_source_reader=_read_data_source,
            data_source_writer=_write_data_source,
            staging_location=config.offline_store.staging_location,
            staging_location_endpoint_override=config.offline_store.staging_location_endpoint_override,
        )

    @staticmethod
    def pull_all_from_table_or_query(
        config: RepoConfig,
        data_source: DataSource,
        join_key_columns: List[str],
        feature_name_columns: List[str],
        timestamp_field: str,
        created_timestamp_column: Optional[str] = None,
        start_date: Optional[datetime] = None,
        end_date: Optional[datetime] = None,
    ) -> RetrievalJob:
        return pull_all_from_table_or_query_ibis(
            config=config,
            data_source=data_source,
            join_key_columns=join_key_columns,
            feature_name_columns=feature_name_columns,
            timestamp_field=timestamp_field,
            created_timestamp_column=created_timestamp_column,
            start_date=start_date,
            end_date=end_date,
            data_source_reader=_read_data_source,
            data_source_writer=_write_data_source,
            staging_location=config.offline_store.staging_location,
            staging_location_endpoint_override=config.offline_store.staging_location_endpoint_override,
        )

    @staticmethod
    def offline_write_batch(
        config: RepoConfig,
        feature_view: FeatureView,
        table: pyarrow.Table,
        progress: Optional[Callable[[int], Any]],
    ):
        offline_write_batch_ibis(
            config=config,
            feature_view=feature_view,
            table=table,
            progress=progress,
            data_source_writer=_write_data_source,
        )

    @staticmethod
    def write_logged_features(
        config: RepoConfig,
        data: Union[pyarrow.Table, Path],
        source: LoggingSource,
        logging_config: LoggingConfig,
        registry: BaseRegistry,
    ):
        write_logged_features_ibis(
            config=config,
            data=data,
            source=source,
            logging_config=logging_config,
            registry=registry,
        )
